@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # vim: set expandtab tabstop=4 shiftwidth=4:
 
-# Copyright (c) 2020 CJ Kucera (cj@apocalyptech.com)
+# Copyright (c) 2020-2021 CJ Kucera (cj@apocalyptech.com)
 # 
 # This software is provided 'as-is', without any express or implied warranty.
 # In no event will the authors be held liable for any damages arising from
@@ -417,7 +417,7 @@ class BL3Profile(object):
         current_custs = self.get_cur_customizations(cust_set)
         missing = cust_set - current_custs
         for cust in missing:
-            self.prof.unlocked_customizations.append(OakProfile_pb2.OakCustomizationSaveGameData(
+            self.prof.unlocked_customizations.append(OakShared_pb2.OakCustomizationSaveGameData(
                 is_new=True,
                 customization_asset_path=cust,
                 ))
@@ -430,7 +430,7 @@ class BL3Profile(object):
         current_custs = self.get_cur_weapon_customizations(cust_dict)
         missing = set(cust_dict.keys()) - current_custs
         for cust in missing:
-            self.prof.unlocked_inventory_customization_parts.append(OakProfile_pb2.OakInventoryCustomizationPartInfo(
+            self.prof.unlocked_inventory_customization_parts.append(OakShared_pb2.OakInventoryCustomizationPartInfo(
                 customization_part_hash=cust,
                 is_new=True,
                 ))
@@ -534,7 +534,7 @@ class BL3Profile(object):
         current_custs = self.get_room_decos()
         missing = set(profile_roomdeco_obj_to_eng.keys()) - current_custs
         for cust in missing:
-            self.prof.unlocked_crew_quarters_decorations.append(OakProfile_pb2.CrewQuartersDecorationItemSaveGameData(
+            self.prof.unlocked_crew_quarters_decorations.append(OakShared_pb2.CrewQuartersDecorationItemSaveGameData(
                 is_new=True,
                 decoration_item_asset_path=cust,
                 ))
@@ -621,7 +621,7 @@ class BL3Profile(object):
         # Now do the rearranging
         del self.prof.unlocked_crew_quarters_decorations[:]
         for obj_path, is_new in new_order:
-            self.prof.unlocked_crew_quarters_decorations.append(OakProfile_pb2.CrewQuartersDecorationItemSaveGameData(
+            self.prof.unlocked_crew_quarters_decorations.append(OakShared_pb2.CrewQuartersDecorationItemSaveGameData(
                 is_new=is_new,
                 decoration_item_asset_path=obj_path,
                 ))
@@ -644,35 +644,124 @@ class BL3Profile(object):
             new_order.append((hashval, is_new))
         del self.prof.unlocked_inventory_customization_parts[:]
         for hashval, is_new in new_order:
-            self.prof.unlocked_inventory_customization_parts.append(OakProfile_pb2.OakInventoryCustomizationPartInfo(
+            self.prof.unlocked_inventory_customization_parts.append(OakShared_pb2.OakInventoryCustomizationPartInfo(
                 customization_part_hash=hashval,
                 is_new=is_new,
                 ))
 
+    def _get_generic_keys(self, key_hash):
+        """
+        The profile holds info for a few different key types now.  This
+        will return the count for the specified `key_hash`.
+        """
+        for cat in self.prof.bank_inventory_category_list:
+            if cat.base_category_definition_hash == key_hash:
+                return cat.quantity
+        return 0
+
+    def _set_generic_keys(self, key_hash, num_keys):
+        """
+        The profile holds info for a few different key types now.  This
+        will set the number of keys for the specified `key_hash` to
+        `num_keys.
+        """
+        for cat in self.prof.bank_inventory_category_list:
+            if cat.base_category_definition_hash == key_hash:
+                cat.quantity = num_keys
+                return
+
+        # If we got here, apparently this profile hasn't seen this key type at all
+        self.prof.bank_inventory_category_list.append(OakShared_pb2.InventoryCategorySaveData(
+            base_category_definition_hash=key_hash,
+            quantity=num_keys
+            ))
+
     def get_golden_keys(self):
         """
         Returns the number of golden keys stored on this profile
         """
-        for cat in self.prof.bank_inventory_category_list:
-            if cat.base_category_definition_hash == goldenkey_hash:
-                return cat.quantity
-        return 0
+        return self._get_generic_keys(goldenkey_hash)
 
     def set_golden_keys(self, num_keys):
         """
         Sets the number of golden keys to `num_keys`
         """
-        for cat in self.prof.bank_inventory_category_list:
-            if cat.base_category_definition_hash == goldenkey_hash:
-                cat.quantity = num_keys
-                return
+        self._set_generic_keys(goldenkey_hash, num_keys)
 
-        # If we got here, apparently this profile hasn't seen golden keys at all
-        self.prof.bank_inventory_category_list.append(OakShared_pb2.InventoryCategorySaveData(
-            base_category_definition_hash=goldenkey_hash,
-            quantity=num_keys
+    def get_diamond_keys(self):
+        """
+        Returns the number of diamond keys stored on this profile
+        """
+        return self._get_generic_keys(diamondkey_hash)
+
+    def set_diamond_keys(self, num_keys):
+        """
+        Sets the number of diamond keys to `num_keys`
+        """
+        self._set_generic_keys(diamondkey_hash, num_keys)
+
+    def get_vaultcard1_keys(self):
+        """
+        Returns the number of Vault Card 1 keys stored on this profile
+        """
+        return self._get_generic_keys(vaultcard1key_hash)
+
+    def set_vaultcard1_keys(self, num_keys):
+        """
+        Sets the number of Vault Card 1 keys to `num_keys`
+        """
+        self._set_generic_keys(vaultcard1key_hash, num_keys)
+
+    def _get_vaultcard_chests(self, vcnum):
+        """
+        Returns the number of Vault Card Chests the user has available to open,
+        for the given Vault Card number.
+        """
+        # Fortunately, even if this is run on an old savegame which doesn't
+        # have the vault_card structure, it'll still be present thanks to our
+        # protobuf stuff.
+        for card_rewards in self.prof.vault_card.vault_card_claimed_rewards:
+            if card_rewards.vault_card_id == vcnum:
+                return card_rewards.vault_card_chests
+        return 0
+
+    def _set_vaultcard_chests(self, vcnum, num_chests):
+        """
+        Sets the number of Vault Card Chests the user has available to open,
+        for the given Vault Card number.
+        """
+        # Fortunately, even if this is run on an old savegame which doesn't
+        # have the vault_card structure, it'll still be present thanks to our
+        # protobuf stuff.  We might just want to fix up a few values in it.
+        if self.prof.vault_card.last_active_vault_card_id == 0:
+            self.prof.vault_card.last_active_vault_card_id = vcnum
+        for card_rewards in self.prof.vault_card.vault_card_claimed_rewards:
+            if card_rewards.vault_card_id == vcnum:
+                card_rewards.vault_card_chests = num_chests
+                return
+        self.prof.vault_card.vault_card_claimed_rewards.append(OakShared_pb2.VaultCardRewardList(
+            vault_card_id=vcnum,
+            vault_card_experience=0,
+            unlocked_reward_list=[],
+            redeemed_reward_list=[],
+            vault_card_chests=num_chests,
+            vault_card_chests_opened=0,
+            vault_card_keys_spent=0,
+            gear_rewards=[],
             ))
 
+    def get_vaultcard1_chests(self):
+        """
+        Returns the number of Vault Card #1 Chests the user has available to open
+        """
+        return self._get_vaultcard_chests(1)
+
+    def set_vaultcard1_chests(self, num_chests):
+        """
+        Sets the number of Vault Card #1 Chests the user has available to open
+        """
+        self._set_vaultcard_chests(1, num_chests)
+
     def fixup_guardian_rank(self, force=True):
         """
         Fixes Guardian Rank, based on the redeemed rewards and available tokens.
