@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # vim: set expandtab tabstop=4 shiftwidth=4:
 
-# Copyright (c) 2020 CJ Kucera (cj@apocalyptech.com)
+# Copyright (c) 2020-2021 CJ Kucera (cj@apocalyptech.com)
 # 
 # This software is provided 'as-is', without any express or implied warranty.
 # In no event will the authors be held liable for any damages arising from
@@ -58,7 +58,11 @@ def main():
     prof = BL3Profile(args.filename)
 
     # Golden Keys
-    print('Golden Keys: {}'.format(prof.get_golden_keys()))
+    print('Keys:')
+    print(' - Golden Keys: {}'.format(prof.get_golden_keys()))
+    print(' - Diamond Keys: {}'.format(prof.get_diamond_keys()))
+    print(' - Vault Card 1 Keys: {}'.format(prof.get_vaultcard1_keys()))
+    print(' - Vault Card 1 Chests: {}'.format(prof.get_vaultcard1_chests()))
 
     # Guardian Rank
     print('Guardian Rank: {}'.format(prof.get_guardian_rank()))
