@@ -1,6 +1,26 @@
 #!/usr/bin/env python3
 # vim: set expandtab tabstop=4 shiftwidth=4:
 
+# Copyright (c) 2020-2021 CJ Kucera (cj@apocalyptech.com)
+# 
+# This software is provided 'as-is', without any express or implied warranty.
+# In no event will the authors be held liable for any damages arising from
+# the use of this software.
+# 
+# Permission is granted to anyone to use this software for any purpose,
+# including commercial applications, and to alter it and redistribute it
+# freely, subject to the following restrictions:
+# 
+# 1. The origin of this software must not be misrepresented; you must not
+#    claim that you wrote the original software. If you use this software in a
+#    product, an acknowledgment in the product documentation would be
+#    appreciated but is not required.
+# 
+# 2. Altered source versions must be plainly marked as such, and must not be
+#    misrepresented as being the original software.
+# 
+# 3. This notice may not be removed or altered from any source distribution.
+
 import io
 import json
 import lzma
@@ -109,7 +129,7 @@ class BL3Serial(object):
         """
 
         self.serial = serial
-        (self.decrypted_serial, self.orig_seed) = BL3Serial._decrypt_serial(serial)
+        (self.decrypted_serial, self.orig_seed, self.serial_version) = BL3Serial._decrypt_serial(serial)
         self.parsed = False
         self.parts_parsed = False
         self.can_parse = True
@@ -130,6 +150,7 @@ class BL3Serial(object):
         self._manufacturer_idx = None
         self._manufacturer = None
         self._level = None
+        self._rerolled = None
         self._remaining_data = None
 
         # Additional data that gets filled in if we can parse parts
@@ -198,8 +219,9 @@ class BL3Serial(object):
         Decrypts (really just de-obfuscates) the serial number.
         """
 
-        # Initial byte should always be 3
-        assert(serial[0] == 3)
+        # Initial byte should always be 3 or, after the 2021-04-08 patch, 4.
+        assert(serial[0] == 3 or serial[0] == 4)
+        serial_version = serial[0]
 
         # Seed does need to be an unsigned int
         orig_seed = struct.unpack('>i', serial[1:5])[0]
@@ -223,10 +245,10 @@ class BL3Serial(object):
                 ))
 
         # Return what we decrypted
-        return (decrypted[2:], orig_seed)
+        return (decrypted[2:], orig_seed, serial_version)
 
     @staticmethod
-    def _encrypt_serial(data, seed=None):
+    def _encrypt_serial(data, serial_ver, seed=None):
         """
         Given an unencrypted `data`, return the binary serial number for
         the item, optionally with the given `seed`.  If `seed` is not passed in,
@@ -239,7 +261,7 @@ class BL3Serial(object):
             seed = random.randrange(0x100000000) - 0x80000000
 
         # Construct our header and find the checksum
-        header = struct.pack('>Bi', 3, seed)
+        header = struct.pack('>Bi', serial_ver, seed)
         crc32 = binascii.crc32(header + b"\xFF\xFF" + data)
         checksum = struct.pack('>H', ((crc32 >> 16) ^ crc32) & 0xFFFF)
 
@@ -322,7 +344,7 @@ class BL3Serial(object):
         self._balance_short = self._balance.split('.')[-1]
 
         # If we know of an English name for this balance, use it
-        self._eng_name = self.name_db.get(self._balance_short)
+        self._eng_name = self.name_db.get(self._balance)
 
         # Mark down that we've parsed the basic info (we have enough to level up
         # gear at this point)
@@ -367,12 +389,20 @@ class BL3Serial(object):
                 self.parts_parsed = False
                 self.can_parse_parts = False
 
+            # If we're a v4 (or higher) serial, read in the number of times we've
+            # been re-rolled
+            if self.serial_version >= 4:
+                self._rerolled = bits.eat(8)
+            else:
+                self._rerolled = 0
+
             # And read in our remaining data.  If there's more than 7 bits
             # left, we've done something wrong, because it should only be
             # zero-padding after all the "real" data is in place.
             if len(bits.data) > 7:
                 self.parts_parsed = False
                 self.can_parse_parts = False
+                pass
             elif '1' in bits.data:
                 # This is supposed to only be zero-padding at the moment, if
                 # we see something else, abort
@@ -443,6 +473,10 @@ class BL3Serial(object):
             # Then our number of customs (should always be zero)
             bits.append_value(self._num_customs, 4)
 
+            # Then, if we're a v4 serial, the number of times we've been rerolled
+            if self.serial_version >= 4:
+                bits.append_value(self._rerolled, 8)
+
         else:
             # Otherwise, we can re-use our original remaining data
             bits.append_data(self._remaining_data)
@@ -451,7 +485,7 @@ class BL3Serial(object):
         new_data = bits.get_data()
 
         # Encode the new serial (using seed 0; unencrypted)
-        new_serial = BL3Serial._encrypt_serial(new_data, 0)
+        new_serial = BL3Serial._encrypt_serial(new_data, self.serial_version, 0)
 
         # Load in the new serial (this will set `parsed` to `False`)
         # It bothers me that I've just done an `_encrypt_serial` in the
@@ -536,7 +570,7 @@ class BL3Serial(object):
             seed = self.orig_seed
         else:
             seed = 0
-        return BL3Serial._encrypt_serial(self.decrypted_serial, seed)
+        return BL3Serial._encrypt_serial(self.decrypted_serial, self.serial_version, seed)
 
     def get_serial_base64(self, orig_seed=False):
         """
@@ -805,9 +839,8 @@ class InventorySerialDB(object):
 
 class BalanceToName(object):
     """
-    Little wrapper to provide access to a mapping from Balance names (actually
-    just the "short" version of those, without path) to English names that
-    we can report on.
+    Little wrapper to provide access to a mapping from Balance names to
+    English names that we can report on.
     """
 
     def __init__(self):
@@ -821,7 +854,7 @@ class BalanceToName(object):
         """
         if not self.initialized:
             with lzma.open(io.BytesIO(pkg_resources.resource_string(
-                    __name__, 'resources/short_name_balance_mapping.json.xz'
+                    __name__, 'resources/balance_name_mapping.json.xz'
                     ))) as df:
                 self.mapping = json.load(df)
             self.initialized = True
@@ -832,10 +865,8 @@ class BalanceToName(object):
         """
         if not self.initialized:
             self._initialize()
-        if '/' in balance:
-            balance = balance.split('/')[-1]
         if '.' in balance:
-            balance = balance.split('.')[-1]
+            balance = balance.rsplit('.', 1)[0]
         balance = balance.lower()
         if balance in self.mapping:
             return self.mapping[balance]
